/*
 * ReportForm.java
 *
 * Copyright (C) 2005 Tommi Laukkanen
 * http://www.substanceofcode.com
 *
 * Created on January 5th 2005
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

package com.substanceofcode.carmanager.view;

import com.substanceofcode.carmanager.controller.CarManagerMIDlet;
import javax.microedition.lcdui.*;
import java.util.*;
import com.substanceofcode.carmanager.model.SubDate;
import com.substanceofcode.carmanager.model.FillupEntry;

/**
 *
 * @author Tommi Laukkanen
 */
/**
 *  Information form
 */
public class ReportForm extends Form implements CommandListener {
    
    private CarManagerMIDlet m_midlet = null;
    
    private Command m_backCommand = new Command("Back", Command.BACK, 1);
    private Command m_timeSpanCommand = new Command("Time span", Command.SCREEN, 2);
    private Image   m_spacer = null;
    
    /** 
     * Constructor adds two commands to the form and 
     * initializes the spacer image
     */
    public ReportForm(CarManagerMIDlet midlet){
        super(null);
        m_midlet = midlet;
        addCommand( m_backCommand );
        addCommand( m_timeSpanCommand );
        setCommandListener(this);
        m_spacer = loadImage("/icons/space.png");
    }
    
    /** Load an image */
    private Image loadImage(String filename) {
        System.out.println("Loading image: " + filename);
        Image image = null;
        try {
            image = Image.createImage(filename);
        } catch(Exception e) {
            System.out.println("Error while loading image: " + filename);
            System.out.println("Description: " + e.toString());
            // Use null
        }
        return image;
    }    
    
    public void refresh() {
        
        long lastKm=0;
        long litresSinceLastFullFillup=0;
        long averageConsumption=0;
        long averageLastConsumption=0;
        long latestAverageConsumption=0;
        long lngFullFillUpCounter=0;
        int  fillupCount = 0;

        int  m_totalFillUps = 0;
        long m_totalPrice = 0;
        int  m_averagePricePerLitre = 0;
        int  m_averageFillup = 0;
        int  m_litresPerKm = 0;        
        
        Date start = m_midlet.getReportStartDate();
        Date end   = m_midlet.getReportEndtDate();
        
        /**
         * Loop through entries and calculate following:
         * 1. average fill-up
         * 2. average consumption (since last full fill-up)
         * 3. average consumption (total)
         * 4. average price per litre
         * 5. Total fill-ups
         * 6. Total price
         */
        Enumeration IDs = m_midlet.getFillups().elements();
        while (IDs.hasMoreElements()) {
            FillupEntry tank = (FillupEntry) IDs.nextElement();
            
            /** Check if fill-up matches the time span */
            if( tank.isNewer( start ) && 
                tank.isOlder( end   ) ) {
                
                /** Count fill-ups */
                fillupCount++;
                
                /** Calculate total fill-ups */
                m_totalFillUps += tank.getGas();
                
                /** Calculate total price */
                m_totalPrice += tank.getPrice();
                
                /** Calculate average price per litre */
                if( tank.getGas()>0)
                    m_averagePricePerLitre += tank.getPrice() / (tank.getGas()/100);
                
                /** Calculate average consumptions */
                if( tank.getFullFillUp()==1 && lastKm!=0 ) {
                    averageConsumption += averageLastConsumption;
                    
                    averageLastConsumption =
                            (litresSinceLastFullFillup*100)/(lastKm - tank.getKm());
                    
                    if( latestAverageConsumption == 0 )
                        latestAverageConsumption = averageLastConsumption;
                    
                    lngFullFillUpCounter++;
                    litresSinceLastFullFillup = 0;//tank.getGas();
                    lastKm = tank.getKm();
                }
                if( tank.getFullFillUp()==1 && lastKm==0 ) {
                    
                    litresSinceLastFullFillup = 0;//tank.getGas();
                    lastKm = tank.getKm();
                    
                }
                if( tank.getFullFillUp()!=1 ) {
                    
                }
                
                litresSinceLastFullFillup += tank.getGas();
            }
        }// End while

                            
        if( averageLastConsumption>0 ) {
            averageConsumption += averageLastConsumption;
        }
        
        /** Calculate average fill-ups and price per litre */
        if( fillupCount>0) {
            m_averageFillup = m_totalFillUps / fillupCount;
            m_averagePricePerLitre = m_averagePricePerLitre / fillupCount;
        }
        
        if( lngFullFillUpCounter>0 ) {
            averageConsumption = averageConsumption / lngFullFillUpCounter;
        }
        
        while(size()>0)
            delete(0);
        
        Calendar c = Calendar.getInstance();
        c.setTime(start);
        
        SubDate startDate = new SubDate( start );
        SubDate endDate   = new SubDate( end );
        
        append(new StringItem("Report\n",
                startDate.getYear() + "/" + startDate.getMonth() + 
                "/" + startDate.getDay() +
                " to " + endDate.getYear() + "/" + endDate.getMonth() + 
                "/" + endDate.getDay()));
        append(m_spacer);
        append(new StringItem("Fill-up count:", Integer.toString(fillupCount)));
        append(m_spacer);
        append(new StringItem("Average fill-up:", m_midlet.Cents2Normal(Integer.toString(m_averageFillup)) + "l"));
        append(m_spacer);
        append(new StringItem("Average consumption:", m_midlet.Cents2Normal(Long.toString(averageConsumption)) + " l/100km"));
        append(m_spacer);
        append(new StringItem("Average consumption since last full fill-up:", m_midlet.Cents2Normal(Long.toString(latestAverageConsumption)) + " l/100km"));
        append(m_spacer);
        append(new StringItem("Average price:", m_midlet.Cents2Normal(Integer.toString(m_averagePricePerLitre)) + " /l"));
        append(m_spacer);
        append(new StringItem("Total fill-ups:", m_midlet.Cents2Normal(Integer.toString(m_totalFillUps)) + "l"));
        append(m_spacer);
        append(new StringItem("Total price:", m_midlet.Cents2Normal(Long.toString(m_totalPrice)) + ""));
    }
    
    public void commandAction(Command c, Displayable s) {
        if( c == m_backCommand ) {
            m_midlet.showFillups();
        }
        if( c == m_timeSpanCommand ) {
            m_midlet.showTimeSpan();
        }
    }
}
